#ifdef MODBUS_SUPPORT_ENABLED
#include "iemdb.h"
#include <modbus.h>
//Modbus max data size definition (byte counting)
#define MAX_WRITE_DATA_SIZE MODBUS_MAX_WRITE_REGISTERS*2
#define MAX_READ_DATA_SIZE  MODBUS_MAX_READ_REGISTERS*2

/* ---------------------------------------------------------*/
/* FUNCTIONS CODE											*/
/* ---------------------------------------------------------*/

/*----------------------------------------------------------*/
/* Local functions                                          */
/* ---------------------------------------------------------*/

/* Function to load a struct timeval correctly
 * from a double.
 */
static inline struct timespec d_to_timespec(double time) {
  struct timespec tmp;

  tmp.tv_sec  = time;
  tmp.tv_nsec = 1e9*(time - tmp.tv_sec);
  return tmp;
}

/*----------------------------------------------------------*/
/* Glocal functions                                         */
/* ---------------------------------------------------------*/

/*----------------------------------------------------------*/
/* Time funtion
 * IeMdbSetTime: Convert lynxos time_t format to PLC SCHNEIDER
 * format.
 *
 * Details:
 * PLC SCHNEIER format is coded on 8 bytes BCD
 * format:
 *
 *  0      1      2      3      4      5     6     7
 *  SC     00     HH     MN     MM     DD    YY	   YY
 *
 *  byte 1 is used by gateway to synchronize time setting.
 */
void IeMdbSetTime(unsigned char *dt, time_t epoch)  // to PLC
{
  struct tm *dscst;
  dscst = localtime(&epoch);

  /*v1.7*/
  /*In fact, each 16bits word is swapped with SCHNEIDER PLC
    1      0      3      2      5      4     7     6
    SC     00     HH     MN     MM     DD    YY	   YY

	Swapping can do it directly during data transferring
  */

  dt[1] = _tobcd(dscst->tm_sec);
  dt[0] = 0x00; /*not used*/
  dt[3] = _tobcd(dscst->tm_hour);
  dt[2] = _tobcd(dscst->tm_min);
  dt[5] = _tobcd(dscst->tm_mon+1);
  dt[4] = _tobcd(dscst->tm_mday);
  dt[7] = _tobcd(2000/100);
  dt[6] = _tobcd(dscst->tm_year-100);
}

/*----------------------------------------------------------*/
/* Time funtion
 * IeMdbGetTime: Convert PLC SCHNEIDER format to time_t format.
 *
 * Details:
 * PLC SCHNEIER format is coded on 8 bytes BCD
 * format:
 *
 *  0      1        2      3      4      5     6     7
 *  SC     SC/100   HH     MN     MM     DD    YY	 YY
 *
 */
double IeMdbGetTime(unsigned char *dt) // from PLC
{
  struct tm plcst;
  time_t plctm;
  double ms;
  int year;

  /*In fact, each 16bits word is swapped with SCHNEIDER PLC
    1      0        3      2      5      4     7     6
    SC     SC/100   HH     MN     MM     DD    YY	 YY

	Swapping can do it directly during data transferring
  */

  plcst.tm_sec  = _frombcd(dt[1]);
  plcst.tm_hour = _frombcd(dt[3]);
  plcst.tm_min  = _frombcd(dt[2]);
  plcst.tm_mon  = _frombcd(dt[5])-1;
  plcst.tm_mday = _frombcd(dt[4]);
  year = _frombcd(dt[6]);
  //look at Schneider DATA_AND_TIME type documentation
  plcst.tm_year = ((year >= 90) ? year : year+100);
  /*plcst.tm_wday = no used in PLC*/
  /*plcst.tm_yday = no used in PLC*/

  ms = ((double)_frombcd(dt[0])/100.);
  plcst.tm_isdst = -1; // daylight saving time unavailable
  plctm = mktime(&plcst);

  return((double)plctm + ms);
}

/*----------------------------------------------------------*/
int IeMBwriteData(modbus_t* ctx, uint16_t start_addr, uint16_t count, uint8_t* data)
{
    uint16_t *srcp = (uint16_t*)data;
    int word_count, byte_count;

    while(count)
    {
        word_count = (count > MAX_WRITE_DATA_SIZE) ? MAX_WRITE_DATA_SIZE/2 : (count/2)+(count%2);
        byte_count = (word_count * 2);

        if (modbus_write_registers(ctx, start_addr, word_count, srcp) != word_count)
        {
            return IE_EPIPE_ERROR;
        }

        //srcp += byte_count-(count%2);
        srcp += word_count;
        start_addr += word_count;
        count -= (byte_count - (count%2));
    }

    return 0;
}

/*----------------------------------------------------------*/
int IeMBreadData(modbus_t* ctx, uint16_t start_addr, uint16_t count, uint8_t* data)
{
  uint16_t *destp = (uint16_t*)data;
  int word_count, byte_count;

  while(count) {

      if (count > MAX_READ_DATA_SIZE)
      {
         /*'word_count' is a word counter*/
         word_count = MAX_READ_DATA_SIZE/2;
      }
      else
      {
        /*'word_count' is a word counter*/
        word_count = (count/2) + (count%2);
      }

      byte_count = (word_count * 2);

      if (modbus_read_registers(ctx, start_addr, word_count, destp) != word_count)
      {
          return IE_EPIPE_ERROR;
      }

      //destp += (byte_count-(count%2));
      destp += word_count;
      start_addr += word_count;
      count -= (byte_count - (count%2));
  }

  return 0;
}
#endif //MODBUS_SUPPORT_ENABLED
